package smail.filter;

/** This class represents a Filter <P>
 * A filter contains a Condition, which holds the information when a filter is applied, and an <b>action</b>,
 * which <br> specifies the action is applied.
 * @see smail.filter.Action
 * @see smail.filter.Condition
 */
public class Filter {
    
    /** constant for type-declaration of the actual filter<br>
     * This constant is used for pre-filter, means the filter is
     * <b>automatically</b> applied <b>before</b> receiving the mails.
     */ 
    public static final int PRE       = 0;
    
    /** constant for type-declaration of the actual filter<br>
     * This constant is used for post-filter, means the filter is 
     * <b>automatically</b> applied <b>after</b> receiving the mails.
     */
    public static final int POST      = 1;
    
    /** constant for type-declaration of the actual filter<br>
     * This constant is used for manually-filter, means the filter is applied
     * <b>manually</b> by the user.
     */
    public static final int MANUALLY  = 2;
    
    /** constant for type-declaration of the actual filter<br>
     * This constant is used for filter, which are applied <b>automatically
     * after receiving</b> but can also applied <b>manually</b> by the user.
     */
    public static final int REC_MAN   = 3;
    
    /** should be set of one of the constants, which are explained above*/
    private int                 type;
    
    /** the name of the filter*/
    private java.lang.String    name ;
    
    /** the filterID of the filter (the number in the property-file) */
    private int filterId;
    
    /** is filter active or not ?*/
    private boolean             isActive; 
    
    /** the Action of the filter<br> 
     * @see smail.filter.Action
     */
    private Action      action;
    
    /** the Condition of the filter<br>
     * @see smail.filter.Condition
     */
    private Condition[]   condition;

    /** creates a new filter-object
     * @param type the type of the filter (see above)
     * @param filterID the number of the filter in the Property-File
     * @param act the action of the filter (see smail.filter.Action)
     * @param cond the Condition of the filter (see smail.filter.Condition)
     */
     public Filter (int type, String name, smail.filter.Action act, smail.filter.Condition cond[]) throws smail.filter.NoSuchTypeException {
        
        if (type >= 0 && type <= 3)
            this.type = type;
        else throw new smail.filter.NoSuchTypeException();
        
        this.name       = name;
        this.filterId   = filterId;
        this.isActive   = true;
        this.action     = act;
        this.condition  = cond;
    }
    
    /** sets the type of the filter */
    public void setType (int type) throws smail.filter.NoSuchTypeException { 
        if (type >= 0 && type <= 3)
            this.type = type;
        else throw new smail.filter.NoSuchTypeException();
    }
    
    /** to get the type of the filter */
    public int getType () { return this.type; }

    /** sets the name of the filter */
    public void setName (java.lang.String name) { 
        this.name = name; 
        smail.utils.PropertiesManager.set("smail.filter." + this.getFilterId() + ".name", name);
    }
    
    /** to get the name of the filter */
    public java.lang.String getName () { return this.name; }
    
    /** to get the filterId of the filter */
    public int getFilterId () { return this.filterId; }
    
     /** set the filterId of the filter */
    public void setFilterId (int filterId) { this.filterId = filterId; }
    
    
    
    /** sets the activity of the filter */
    public void setActivity (boolean activity) { 
        this.isActive = activity; 
        smail.utils.PropertiesManager.set("smail.filter." + this.getFilterId() + ".active", java.lang.String.valueOf(activity));
    }
    
    /** to get the activity of the filter */
    public boolean getActivity () { return this.isActive; }
    
    /** sets the action of the filter */
    public void setAction (smail.filter.Action act) { 
        this.action = act ; 
        smail.utils.PropertiesManager.set("smail.filter." + this.getFilterId() + ".action.type", java.lang.String.valueOf(act.getType()) );
        smail.utils.PropertiesManager.set("smail.filter." + this.getFilterId() + ".action.destination", java.lang.String.valueOf(act.getDestination()) );
    }
    
    /** to get the action of the filter */
    public smail.filter.Action getAction() { return this.action ; }
    
    /** sets the conditions of the filter */
    public void setCondition (smail.filter.Condition[] cond) { 
        this.condition =  cond;
        
        //get the new count of conditions
        int newCount = cond.length;

        //get the old count of conditions
        int oldCount = 0;
        try { oldCount = java.lang.Integer.parseInt(smail.utils.PropertiesManager.get("smail.filter." + filterId + ".condition.count")); }
        catch (NumberFormatException e) { System.out.println("NumberFormatExceotion in Filter" + this.getName()); }
        
        for (int i=0; i<newCount;i++) {
            
            // which type of condition
            if (cond[i] instanceof SubjectCondition) {
                smail.utils.PropertiesManager.set("smail.filter." + this.filterId + ".condition." + i + ".keyword", ((FromCondition)cond[i]).getKeyword() );
                smail.utils.PropertiesManager.set("smail.filter." + this.filterId + ".condition." + i + ".dateType", "");
                smail.utils.PropertiesManager.set("smail.filter." + this.filterId + ".condition." + i + ".date", "");
            }
            else if (cond[i] instanceof FromCondition) {
                smail.utils.PropertiesManager.set("smail.filter." + this.filterId + ".condition." + i + ".keyword", ((FromCondition)cond[i]).getKeyword() );
                smail.utils.PropertiesManager.set("smail.filter." + this.filterId + ".condition." + i + ".dateType", "");
                smail.utils.PropertiesManager.set("smail.filter." + this.filterId + ".condition." + i + ".date", "");
            }
            else if (cond[i] instanceof DateCondition) {
                smail.utils.PropertiesManager.set("smail.filter." + this.filterId + ".condition." + i + ".keyword", "" );
                smail.utils.PropertiesManager.set("smail.filter." + this.filterId + ".condition." + i + ".dateType", java.lang.String.valueOf(((DateCondition)cond[i]).getType()) );
                smail.utils.PropertiesManager.set("smail.filter." + this.filterId + ".condition." + i + ".date", ((DateCondition)cond[i]).getDate().toString() );
            }
        }
        
        while(newCount < oldCount) {
            smail.utils.PropertiesManager.set("smail.filter." + this.filterId + ".condition." + (newCount+1) + ".keyword", "");
            smail.utils.PropertiesManager.set("smail.filter." + this.filterId + ".condition." + (newCount+1) + ".dateType", "");
            smail.utils.PropertiesManager.set("smail.filter." + this.filterId + ".condition." + (newCount+1) + ".date", "");
            newCount++;
        }
    }
    
    /** to get the conditions of the filter */
    public smail.filter.Condition[] getCondition() { return this.condition ; }
}
