package smail.gui;

import java.util.*;

/** This class contains all information of a mail message. These are:<br>
  * <li> Sender
  * <li> Addressees
  * <li> Addressees (Carbon Copy)
  * <li> Addressees (Blind Carbon Copy)
  * <li> Date of a sent message
  * <li> Subject
  * <li> Header
  * <li> Text
  * <li> Attachments
  * <li> Encryption information
  * <br><br>
  * It has also the information, if the optional properties of a message are set,
  * like Attachments, Cc-Addressees and Bcc-Addressees.
  * If a message contains such element the concerning boolean will be set true,
  * otherwise false.
  */

public class MessageContent
{
    private boolean  bCcAddresses = false;  // false: doesnt exit ccAddresses
    private boolean  bBccAddresses = false;  // false: doesnt exit bccAddresses
    private boolean  bAttachments = false;  // false: doesnt exit Attachments
    private boolean  bEncrypt = false;  // false: Message will not be encrypted

    private String  sFrom         = new String();
    private String  sDate         = new String();
    private Vector  vToAddresses  = new Vector();
    private Vector  vCcAddresses  = new Vector();	// optional
    private Vector  vBccAddresses = new Vector();	// optional

    private String  sSubject      = new String();
    private String  sHeader	  = new String();
    private String  sText         = new String();
    private Vector  vAttachments  = new Vector();	// optional


    // general constructor
    /** Creates a message content with all information
     *  @param sFrom sender of a Message
     *  @param sDate date of a sent message
     *  @param vToAddresses direct addressees
     *  @param vCcAddresses addressees who get a carbon copy
     *  @param vBccAddresses addressees who get a blind carbon copy
     *  @param sSubject subject of a message
     *  @param sHeader header of a message
     *  @param sText text of a message
     *  @param vAttachments attachments to this message
     *  @param bCcAddresses information if cc-addressees exists
     *  @param bBccAddresses information if bcc-addressees exists
     *  @param bAttachments information if attachments exists
     */
    public MessageContent(String sFrom, String sDate,Vector vToAddresses, Vector vCcAddresses,Vector vBccAddresses, String sSubject, String sHeader, String sText, Vector vAttachments, boolean  bCcAddresses, boolean  bBccAddresses, boolean  bAttachments)
    {
    	this.sFrom = sFrom;
    	this.sDate = sDate;

    	this.vToAddresses = (Vector)vToAddresses.clone();
    	this.vCcAddresses = (Vector)vCcAddresses.clone();
    	this.vBccAddresses = (Vector)vBccAddresses.clone();

    	this.sSubject = sSubject;
    	this.sHeader = sHeader;
    	this.sText = sText;
    	this.vAttachments = (Vector)vAttachments.clone();

    	this.bCcAddresses = bCcAddresses;
    	this.bBccAddresses = bBccAddresses;
    	this.bAttachments = bAttachments;
    }

    // special constructor for Editorresult without date and header information
    /** Creates a message content without information about date and header. It is used
     * to store the result of a new written message
     *  @param sFrom sender of a Message
     *  @param vToAddresses direct addressees
     *  @param vCcAddresses addressees who get a carbon copy
     *  @param vBccAddresses addressees who get a blind carbon copy
     *  @param sSubject subject of a message
     *  @param sText text of a message
     *  @param vAttachments attachments to this message
     *  @param bCcAddresses information if cc-addressees exists
     *  @param bBccAddresses information if bcc-addressees exists
     *  @param bAttachments information if attachments exists
     */

    public MessageContent(String sFrom,Vector vToAddresses, Vector vCcAddresses, Vector vBccAddresses, String sSubject, String sText, Vector vAttachments, boolean  bCcAddresses, boolean bBccAddresses, boolean  bAttachments)
    {
    	this.sFrom = sFrom;

    	this.vToAddresses = (Vector)vToAddresses.clone();
    	this.vCcAddresses = (Vector)vCcAddresses.clone();
    	this.vBccAddresses = (Vector)vBccAddresses.clone();

    	this.sSubject = sSubject;
    	this.sText = sText;
    	this.vAttachments = (Vector)vAttachments.clone();

    	this.bCcAddresses = bCcAddresses;
    	this.bBccAddresses = bBccAddresses;
    	this.bAttachments = bAttachments;
    }


    // special constructor for reply
    /** Creates a message content for reply without bccAddresses, header, Attachments and information about optional properties
     *  @param sFrom sender of a Message
     *  @param sDate date of a sent message
     *  @param vToAddresses direct addressees
     *  @param vCcAddresses addressees who get a carbon copy
     *  @param sSubject subject of a message
     *  @param sText text of a message
     */
    public MessageContent(String sFrom, String sDate,Vector vToAddresses, Vector vCcAddresses, String sSubject, String sText)
    {
    	this.sFrom = sFrom;
    	this.sDate = sDate;

    	this.vToAddresses = (Vector)vToAddresses.clone();
    	this.vCcAddresses = (Vector)vCcAddresses.clone();

    	this.sSubject = sSubject;
    	this.sText = sText;
    }



    // special constructor for forward
    /** Creates a message content for forward without from, date,all kind of Addressees and information about optional properties
     *  @param sSubject subject of a message
     *  @param sHeader header of a message
     *  @param sText text of a message
     *  @param vAttachments attachments to this message
     */

    public MessageContent(String sSubject, String sHeader, String sText, Vector vAttachments)
    {
    	this.sSubject = sSubject;
    	this.sHeader = sHeader;
    	this.sText = sText;
    	this.vAttachments = (Vector)vAttachments.clone();
    }


    // Get - Methods

    /** returns the sender of a message, that means the e-mail-address of the person who sent this message
     *	@return sender of a message
     *  @see MessageContent#setFrom
     */
    public String getFrom()
    {
        return sFrom;
    }


    /** returns the date of a message, that means the date and time, when the message was sent
     *	@return date of a message
     *  @see MessageContent#setDate
     */
    public String getDate()
    {
        return sDate;
    }


    /** returns a list of the addressees who will get this message
     *	@return direct addressees
     *  @see MessageContent#setToAddresses
     */
    public Vector getToAddresses()
    {
        return vToAddresses;
    }


    /** returns a list of the addressees who will get this message as carbon copy
     *	@return cc addressees
     *  @see MessageContent#setCcAddresses
     */
    public Vector getCcAddresses()
    {
        return vCcAddresses;
    }


    /** returns a list of the addressees who will get this message as blind carbon copy
     *	@return bcc addressees
     *  @see MessageContent#setBccAddresses
     */
    public Vector getBccAddresses()
    {
        return vBccAddresses;
    }


    /** returns the subject of a message, that could be a short information concerning this message
     *	@return subject of a message
     *  @see MessageContent#setSubject
     */
    public String getSubject()
    {
        return sSubject;
    }


    /** returns the header of a message, that contains more information about the message
     *	@return header of a message
     *  @see MessageContent#setHeader
     */
    public String getHeader()
    {
        return sHeader;
    }


    /** returns the text of a message, the general written part of a message
     *	@return text of a message
     *  @see MessageContent#setText
     */
    public String getText()
    {
        return sText;
    }


    /** returns a list of attachments added to a message
     *	@return attachments of a message
     *  @see MessageContent#setAttachments
     */
    public Vector getAttachments()
    {
        return vAttachments;
    }


    /** returns the information if this message has a list of cc-addressees. It returns true, if there are cc-addressees, otherwise it returns false
     *	@return information about cc-addressees
     *  @see MessageContent#setCcAddressesBoolean
     */
    public boolean getCcAddressesBoolean()
    {
        return bCcAddresses;
    }


    /** returns the information if this message has a list of bcc-addressees. It returns true, if there are bcc-addressees, otherwise it returns false
     *	@return information about bcc-addressees
     *  @see MessageContent#setBccAddressesBoolean
     */
    public boolean getBccAddressesBoolean()
    {
        return bBccAddresses;
    }


    /** returns the information if this message has a list of attachments.
     *  It returns true, if there are attachments added to this message, otherwise it returns false
     *	@return information about attachments
     *  @see MessageContent#setAttachmentsBoolean
     */
    public boolean getAttachmentsBoolean()
    {
        return bAttachments;
    }


    /** returns the information if this message should be encrypted.
     *  It returns true, if encryption is desired, otherwise it returns false
     *	@return information about encryption
     *  @see MessageContent#setEncryptBoolean
     */
    public boolean getEncryptBoolean()
    {
        return bEncrypt;
    }





    // Set - Methods



    /** sets the sender of a message, that means the e-mail-address of the person who sent this message
     *	@param sender of a message
     *  @see MessageContent#getFrom
     */
    public void setFrom(String sNewFrom)
    {
        sFrom = sNewFrom;
    }


    /** sets the date of a message, that means the date and time, when the message was sent
     *	@param date of a message
     *  @see MessageContent#getDate
     */
    public void setDate(String sNewDate)
    {
        sDate = sNewDate;
    }


    /** sets a list of the addressees who will get this message
     *	@param a list of the direct addressees
     *  @see MessageContent#getToAddresses
     */
    public void setToAddresses(Vector vNewToAddresses)
    {
        vToAddresses = (Vector) vNewToAddresses.clone();
    }


    /** sets a list of the addressees who will get this message as carbon copy
     *	@param a list of the cc addressees
     *  @see MessageContent#getCcAddresses
     */
    public void setCcAddresses(Vector vNewCcAddresses)
    {
        vCcAddresses = (Vector) vNewCcAddresses.clone();
    }


    /** sets a list of the addressees who will get this message as blind carbon copy
     *	@param a list of the bcc addressees
     *  @see MessageContent#getBccAddresses
     */
    public void setBccAddresses(Vector vNewBccAddresses)
    {
        vBccAddresses = (Vector) vNewBccAddresses.clone();
    }



    /** sets the subject of a message, that could be a short information concerning this message
     *	@param subject of a message
     *  @see MessageContent#getSubject
     */
    public void setSubject(String sNewSubject)
    {
        sSubject = sNewSubject;
    }



    /** sets the header of a message, that contains more information about the message
     *	@param header of a message
     *  @see MessageContent#getHeader
     */
    public void setHeader(String sNewHeader)
    {
        sHeader = sNewHeader;
    }


    /** sets the text of a message, the general written part of a message
     *	@param text of a message
     *  @see MessageContent#getText
     */
    public void setText(String sNewText)
    {
        sText = sNewText;
    }



    /** sets a list of attachments added to a message
     *	@param a list of attachments
     *  @see MessageContent#getAttachments
     */
    public void setAttachments(Vector vNewAttachments)
    {
        vAttachments = (Vector) vNewAttachments.clone();
    }

    /** sets a list of attachments added to a message
     *	@param a list of attachments
     *  @see MessageContent#getAttachments
     */
    public void addAttachment(Object o)
    {
        vAttachments.addElement(o);
    }



    /** sets the information if this message has a list of cc-addressees. Set true, if there are cc-addressees, otherwise set false
     *	@param information about cc-addressees
     *  @see MessageContent#getCcAddressesBoolean
     */
    public void setCcAddressesBoolean(boolean bNewCcAddresses)
    {
        bCcAddresses = bNewCcAddresses;
    }


    /** sets the information if this message has a list of bcc-addressees. Set true, if there are bcc-addressees, otherwise set false
     *	@param information about bcc-addressees
     *  @see MessageContent#getBccAddressesBoolean
     */
    public void setBccAddressesBoolean(boolean bNewBccAddresses)
    {
        bBccAddresses = bNewBccAddresses;
    }


    /** sets the information if this message has a list of attachments. Set true, if there are attachments, otherwise set false
     *	@param information about attachments
     *  @see MessageContent#getAttachmentsBoolean
     */
    public void setAttachmentsBoolean(boolean bNewAttachment)
    {
        bAttachments = bNewAttachment;
    }


    /** sets the information if this message should be encrypted. Set true, if encryption is desired, otherwise set false
     *	@param information about encryption
     *  @see MessageContent#getEncryptBoolean
     */
    public void setEncryptBoolean(boolean bNewEncrypt)
    {
        bEncrypt = bNewEncrypt;
    }

}
