/*
 * localFolder.java
 *
 * Created on 25. Mai 2001, 17:21
 */

package smail.localStore;

/** This class represents a folder for mail messages stored locally in mbox
 * format. <P>
 *
 * <CODE>LocalFolder</CODE>s can contain both, folders and messages, thus they
 * provide a tree-like hierarchy rooted at <CODE>LocalStore</CODE>'s default
 * folder. <P>
 *
 * Foldernames have to be valid filenames, i.e. they are a little bit OS
 * dependent.
 * But they are case sensitive and the hierarchy delimiter is a slash
 * ('<CODE>/</CODE>') on all platforms. <P>
 *
 * A subfolder with the name <B>INBOX</B> is always present as the default
 * folder. <P>
 *
 * So are <B>OUTBOX</B>, <B>DRAFTS</B>, <B>TRASH</B> and <B>SENT</B>.
 *
 * TODO: only some of the methods are delivering events<br>
 * TODO: this class doesn't meet the specs in respect to returning folders:
 *       it returns references instead of new distinct objects<br>
 * 
 * @see "javax.mail.Folder"
 */
public class LocalFolder extends javax.mail.Folder {

  /** flag that holds the status of this folder */
  private boolean isOpen;
  
  /** is set when there are new (RECENT) messages */
  private boolean hasNewMessages;

  /** name of this folder */
  private LocalFolderName name;
  
  /** the messages in this folder */
  private java.util.Vector messages;
  
  private MBox mbox;

  // TODO: write comments for this class
  private class LocalFolderName {
    private String path;
    private String name;
    
    LocalFolderName(String fullName) {
      // split full name into path and name
      int slashPos = fullName.lastIndexOf('/');
      path = fullName.substring(0, slashPos+1);
      name = fullName.substring(slashPos+1);
    }
    
    public String getPath() { return path; }
    
    public String getName() { return name; }
    
    public void setName(String name) { this.name = name; }
    
    public String getFullName() { return path + name; }
    
    public String getParentName() {
      String tmp = path.substring(0,path.length()-1);
      return (tmp.length()==0) ? null : tmp;
    }
    
    public String toString() {
      return "path: " + path + "  name:" + name;
    }
  }
  
  /** Creates new LocalFolder which belongs to a given LocalStore.
   *
   * @param ls the LocalStore this folder belongs to.
   */
  public LocalFolder(LocalStore ls) {
    super(ls);
    
    // init the fields
    isOpen = false;
    hasNewMessages = false;
    messages = new java.util.Vector();
  }
  
/** Create a new Folder. This constructor is called by the LocalStore object.
 * Don't use it to create own folders.
 * @param ls the LocalStore this folder belongs to.
 * @param fullName is the full, i.e. absolute name of this folder
 * @see LocalStore
 
  public LocalFolder(LocalStore ls, String fullName) {
    this(ls);
    name = new LocalFolderName(fullName);
  } */
  
  public LocalFolder(LocalStore ls, String fullName, MBox mbox) {
    this(ls);
    name = new LocalFolderName(fullName);
    this.mbox = mbox;
    this.mbox.setFolder(this);
  }
  
/** Append given messages to this folder. This method can be invoked on a closed
 * folder. An appropriate MessageCountEvent is delivered to any
 * MessageCountListener. <P>
 *
 * @param msgs the messages to add to this folder
 * @throws MessagingException if the append failed
 * @throws FolderNotFoundException if this folder does not exist
 */  
  public void appendMessages(javax.mail.Message[] msgs)
  throws javax.mail.MessagingException, javax.mail.FolderNotFoundException {
    
    // add all messages to the collection
    for (int i=0; i<msgs.length; i++) {
      javax.mail.Message msg = msgs[i];

      // filter expunged messages
      if (!msg.isExpunged()) {
        // convert message to LocalMessage if necessary
        if (!(msg instanceof LocalMessage)) {
          // hope this works because of the "lighweight message" paradigm
          msg = new LocalMessage((javax.mail.internet.MimeMessage)msg);
        }
        // set messagenumber of the message
        ((LocalMessage)msg).setMessageNumber(messages.size()+1);
        
        messages.add(msg);

      }
    }
    try {
      mbox.appendMessages((LocalMessage[])messages.toArray(new LocalMessage[messages.size()]));
    } catch (java.io.IOException ioe) {
      ioe.printStackTrace();
    }

    notifyMessageAddedListeners(msgs);
  }
  
/** Close this folder. This method is only valid on open folders.<p>
 *
 * Note that the folder is closed even if this method terminates abnormally
 * by throwing a <CODE>MessagingException</CODE><p>
 *
 * @param expunge expunges all deleted messages if this flag is true
 * @throws MessagingException -
 * @throws IllegalStateException if this folder is not opened
 */  
  public void close(boolean expunge) throws javax.mail.MessagingException, IllegalStateException {
    // can't close if not open. pretty obvious, huh?
    if (!isOpen()) throw new java.lang.IllegalStateException();

    if (expunge) expunge();
    
    // TODO: write index file to disk and delete lock file
    // TODO: send javax.mail.event.ConnectionEvent to ConnectionListeners
    
    isOpen = false;
    notifyConnectionListeners(javax.mail.event.ConnectionEvent.CLOSED);
  }
  
/** Create this folder on the Store. <P>
 *
 * The type parameter is ignored because all our folders can hold both,
 * messages and (sub)folders.<P>
 *
 * The specs say that all folders in the path to this folder that don't
 * exist, are created automatically. This is not true for this implementation
 * (yet)!
 *
 * TODO: fire FolderEvent<BR>
 *
 * @param type the type of this folder (ignored!)
 * @throws MessagingException -
 * @return true if the creation succeeds, else false.
 */  
  public boolean create(int type) throws javax.mail.MessagingException {
    
    // TODO: create subdirectories and files on disk
    
    return true;
  }
  
/** Create a folder. A convenience method because LocalFolder ignores
 * the parameter anyway.
 * @throws MessagingException -
 * @return true if the creation succeeds, else false.
 */  
  public boolean create() throws javax.mail.MessagingException {
    return create(HOLDS_FOLDERS|HOLDS_MESSAGES);
  }
  
/** FIXME: Maybe broken. Use with care... <P>
 *
 * Delete this folder. This method will only succeed on a closed folder. <p>
 *
 * If <CODE>recurse</CODE> is true, all subfolders will be deleted befor
 * this folder well be deleted. <P>
 *
 * If <CODE>recurse</CODE> is false, the method only succeeds if there are
 * no subfolders. <P>
 *
 * TODO: fire events
 *
 * @param recurse delete all subfolders recursivly if true
 * @throws MessagingException -
 * @throws IllegalStateException if this folder is not in the closed state
 * @throws FolderNotFoundException if this folder does not exist
 * @return true if the folder is deleted successfully
 */  
  public boolean delete(boolean recurse) throws javax.mail.MessagingException,
  IllegalStateException, javax.mail.FolderNotFoundException {
    
    System.out.println("Deleting folder" + name.getFullName());
    
    if (isOpen()) throw new java.lang.IllegalStateException();

    // get a list of all subfolders
//    System.out.println("LocalFolder.delete(): Folder " + this.name.name); // debug
    
    // TODO: test if a simple list() without params would work here too
    javax.mail.Folder[] subfolders = list(name.getFullName() + "/%");
    
    // if recurse is true then delete all subfolders
    if (recurse) {
      for (int i=0; i<subfolders.length; i++) {
        // if deleting a subfolder results in 'false' then return 'false'
        if (!subfolders[i].delete(true)) return false;
      }
    }
    // else test if there are subfolders because we don't allow deleting folders
    // with subfolders
    else if (subfolders.length != 0) return false; // TODO: tell the caller *why* the call failed!?
    
    // if we got this far we need to delete all the messages in this folder
    setFlags(
      (LocalMessage[])messages.toArray(new LocalMessage[messages.size()]),
      new javax.mail.Flags(javax.mail.Flags.Flag.DELETED), true);

    expunge(); // TODO: is this neccesary/desireable here?
    ((LocalStore)store).deleteFolder(this);
    
    return true;
  }
  
/** Test if this folder physically exists on the LocalStore. This method
 * can be invoked on a closed folder. <P>
 *
 * As folders are currently only existent in memory this method returns always
 * true. (Well, actually they exist physically in memory ;-) <P>
 *
 * @throws MessagingException typically if the connection to the server is lost.
 * @return true if the folder exists, otherwise false
 */  
  public boolean exists() throws javax.mail.MessagingException {
    return mbox != null;
  }
  
/** Expunge (permanently remove) messages marked DELTETED. Returns an array
 * containing the expunged Message objects. The <CODE>getMessageNumber()</CODE>
 * method on each of these Message object returns that Message's original
 * (that is, prior to the expunge) sequence number. <P>
 *
 * Expunge causes the renumbering of Message objects subsequent to the
 * expunged messages! <P>
 *
 * After a Message is expunged only the <CODE>isExpunged()</CODE> and
 * <CODE>getMessageNumber()</CODE> are still valid on that Message. Other
 * methods may throw <CODE>MessageRemovedException</CODE>
 *
 * TODO: fire MessageCountEvent<br>
 *
 * @return array of expunged Message objects
 * @throws MessagingException -
 * @throws FolderNotFoundException if this folder does not exist
 * @throws IllegalStateException if this folder is not opened
 */  
  public javax.mail.Message[] expunge() throws javax.mail.MessagingException,
  javax.mail.FolderNotFoundException, IllegalStateException {
    
    java.util.Vector expungedMessages = new java.util.Vector();

    // go through all messages in this folder, remove the messages with the DELETE
    // flag, mark them as 'expunged' and add them to the expungedMessages list
    java.util.Iterator iterator = messages.iterator();
    while (iterator.hasNext()) {
      LocalMessage msg = (LocalMessage)iterator.next();
      if (msg.isSet(javax.mail.Flags.Flag.DELETED)) {
        iterator.remove();
        msg.setExpunged(true);
        expungedMessages.add(msg);
      }
    }
    
    // renumber all messages that are still in this folder
    int i = 1;
    iterator = messages.iterator();
    while (iterator.hasNext()) {
      LocalMessage msg = (LocalMessage)iterator.next();
      msg.setMessageNumber(i++);
    }
    
    // TODO: rewrite mbox or at least index file
    
    return (LocalMessage[])expungedMessages.toArray(new LocalMessage[expungedMessages.size()]);
  }
  
/** Return the LocalFolder object corresponding to the given name. Note that
 * this folder does not physically have to exist in the LocalStore! The
 * <CODE>exists()</CODE> method on a folder indicates wether it really exists
 * on the LocalStore. <P>
 *
 * If the name starts with a slash ('<CODE>/</CODE>') it is interpreded
 * as a full path, otherwise relative to this folder.
 *
 * This method can be invoked on a closed folder. <P>
 *
 * @param name name of the folder
 * @throws MessagingException -
 * @return LocalFolder object
 */  
  public javax.mail.Folder getFolder(String name)
  throws javax.mail.MessagingException {
    
    // if name is not absolute then make it absolute by appending it to the
    // fullname of this folder
    if (!name.startsWith("" + getSeparator()))
      name = (getFullName() + getSeparator() + name);
    
    return (LocalFolder)(store.getFolder(name));
  }
  
/** Returns the full name of this LocalFolder as an absolute path starting with
 * the hierarchy delimiter '<CODE>/</CODE>'. <P>
 *
 * This method can be invoked on a closed folder. <P>
 *
 * @return full name of the folder
 */  
  public String getFullName() { return name.getFullName(); } // return path + getName(); }
  
/** Get the LocalMessage object corresponding to the given message number.
 * Messages are numbered starting at 1 to the total number of messages in the
 * folder. Note that the message number for a particular message can change
 * during a session if other messages are being deleted and the folder is
 * expunged. <P>
 *
 *
 * @param msgnum the message number
 * @throws MessagingException -
 * @throws FolderNotFoundException if this folder does not exist
 * @throws IllegalStateException if this folder is not opened
 * @throws IndexOutOfBoundsException if the message number is out of range
 * @return the LocalMessage object
 */  
  public javax.mail.Message getMessage(int msgnum)
  throws javax.mail.MessagingException, javax.mail.FolderNotFoundException,
  IllegalStateException, IndexOutOfBoundsException {
//    System.out.println("LocalFolder.getMessage(int): " + msgnum); // debug
    // System.exit(0);
    // check if this folder exists, is open and the message number is valid
    if (!exists()) throw new javax.mail.FolderNotFoundException();
    if (!isOpen()) throw new IllegalStateException();
    
    //if (msgnum < 1 || msgnum > messages.size()) throw new IndexOutOfBoundsException(); // dead code
    
    try {
      return mbox.getMimeMessage(msgnum - 1);
    } catch (java.io.IOException ioe) {
      System.out.println("LocalFolder.getMessage(int): IOException on MBox");
      ioe.printStackTrace();
      throw new javax.mail.MessagingException("IOException on MBox");
    }
    // return (LocalMessage)messages.get(msgnum - 1); // dead code
  }
  
/** Get the total number of messages in this LocalFolder. <P>
 *
 * TODO: return the number of messages for closed folders too<br>
 *
 * @throws MessagingException -
 * @throws FolderNotFoundException if this folder does not exist
 * @return total number of messages or -1 if this folder is closed
 */  
  public int getMessageCount() throws javax.mail.MessagingException,
  javax.mail.FolderNotFoundException {
    
    if (!exists()) throw new javax.mail.FolderNotFoundException();

    // i'm a bit lazy here:
    // TODO: rewrite this and return the number of messages of closed folders
    // return (isOpen()) ? messages.size() : -1; // dead code
    return (isOpen()) ? mbox.getMessageCount() : -1;
  }
  
/** Returns the name of this LocalFolder. <P>
 *
 * This method can be invoked on a closed folder. <P>
 * @return name of the LocalFolder
 */  
  public String getName() { return name.getName(); }
  
/** Returns the parent folder of this LocalFolder. This method can be invoked
 * on a closed folder. If this folder is the top of a folder hierarchy this
 * method returns <CODE>null</CODE> <P>
 *
 * @throws MessagingException -
 * @return parent folder
 */  
  public javax.mail.Folder getParent() throws javax.mail.MessagingException {
    String tmp = name.getParentName();
    return (tmp==null) ? null : getFolder(tmp);
  }
  
/** Get the permanent Flags supported by this LocalFolder. Returns a Flags object
 * that contains all the flags supported. <P>
 *
 * @return permanent Flags, or <CODE>null</CODE> if not known
 */  
  public javax.mail.Flags getPermanentFlags() {
    // return a Flags object with all flags set to true
    javax.mail.Flags f = new javax.mail.Flags();
    f.add(javax.mail.Flags.Flag.ANSWERED);
    f.add(javax.mail.Flags.Flag.DELETED);
    f.add(javax.mail.Flags.Flag.DRAFT);
    f.add(javax.mail.Flags.Flag.FLAGGED);
    f.add(javax.mail.Flags.Flag.RECENT);
    f.add(javax.mail.Flags.Flag.SEEN);
    // f.add(javax.mail.Flags.Flag.USER);   do we need user defined flags?
    return f;
  }
  
/** Return the delimiter character that separates this LocalFolder's pathname
 * from the names of immediate subfolders. It is always a slash
 * ('<CODE>/</CODE>') in this Folder implementation. This method can be invoked
 * on a closed folder. <P>
 *
 * @return Hierarchy separator character
 * @throws FolderNotFoundException if this folder does not exist
 * @throws MessagingException -
 */  
  public char getSeparator() throws javax.mail.MessagingException,
  javax.mail.FolderNotFoundException {
    
    if (!exists()) throw new javax.mail.FolderNotFoundException();
    return '/';
  }
  
/** Returns the type of this LocalFolder, that is, wether this folder can
 * hold messages or subfolder or both. The returned value is an integer
 * bitfield with the appropriate bits set. This method can be invoked on
 * a closed folder.
 *
 * @throws MessagingException -
 * @throws FolderNotFoundException if this folder does not exist
 * @return integer with appropriate bits set
 */  
  public int getType() throws javax.mail.MessagingException,
  javax.mail.FolderNotFoundException {
    
    if (!exists()) throw new javax.mail.FolderNotFoundException();
    return (HOLDS_FOLDERS|HOLDS_MESSAGES);
  }
  
/** Returns true if this LocalFolder has new LocalMessages since the last time
 * this indication was reset. This indication is set when the folder contains
 * messages with the RECENT flag set. <P>
 *
 * @return true if this LocalFolder contains new messages
 * @throws MessagingException -
 * @throws FolderNotFoundException if this folder does not exist
 */  
  public boolean hasNewMessages() throws javax.mail.MessagingException,
  javax.mail.FolderNotFoundException {
    
    if (!exists()) throw new javax.mail.FolderNotFoundException();

    boolean found = false;
    
    // remember the state of the folder and open it if necessary
    boolean closed = !isOpen();
    if (closed) open(READ_ONLY);
    
    // check the messages for the RECENT flag and set 'found' to true if there's
    // a recent message
    java.util.Iterator iterator = messages.iterator();
    while (iterator.hasNext()) {
      LocalMessage msg = (LocalMessage)iterator.next();
      if (msg.isSet(javax.mail.Flags.Flag.RECENT)) {
        found = true;
        break;
      }
    }

    // close the folder if it was closed before
    if (closed) close(false);
    
    return found;
  }
  
/** Indicates wether this LocalFolder is in the 'open' state
 *
 * @return true if this LocalFolder is in the 'open' state
 */  
  public boolean isOpen() {
    return isOpen;
  }

/** Returns a list of all LocalFolders that are matching the given pattern.
 * <P>
 *
 * Imlementation is not tested at all!!!
 *
 * @param pattern
 * @return null
 * @throws MessagingException -
 */
  public javax.mail.Folder[] list(String pattern)
  throws javax.mail.MessagingException , javax.mail.FolderNotFoundException {
    
//    System.out.println("LocalFolder.list(String): " + pattern + " this.Folderpath: " + name.getFullName()); // debug

    if (!exists()) throw new javax.mail.FolderNotFoundException();

    // make pattern absolut if it was given relative to this folder
    if (pattern.charAt(0) != '/') pattern = name.getFullName() + pattern;
    
    // choose search method and remove wildcard
    int searchType = 0;
    char wildcard = pattern.charAt(pattern.length()-1);
    switch (wildcard) {
      case '*':
        searchType = 1;
        pattern = pattern.substring(0,pattern.length()-1);
        break;
      case '%':
        searchType = 2;
        pattern = pattern.substring(0,pattern.length()-1);
        break;
    }
//    System.out.println("adjusted list pattern: " + pattern); // debug
    
    java.util.LinkedList tmp = new java.util.LinkedList();
    java.util.Iterator iterator = ((LocalStore)store).getFolderIterator();
    while (iterator.hasNext()) {
      String folderName = (String)iterator.next();
      // System.out.println("list folder?: " + folderName); // debug
      if (folderName.startsWith(pattern) && (folderName.length() != pattern.length())) {
        switch (searchType) {
          case 1:
            tmp.add(getFolder(folderName));
            break;
          case 2:
//            System.out.println("x" + folderName + " : " + folderName.indexOf('/', pattern.length()+1)); // debug
            if (folderName.indexOf('/', pattern.length() + 1) == -1) {
              tmp.add(getFolder(folderName));
              // System.out.println(folderName); // debug
            }
            break;
        }
      }
    }
    
    // debug
/*    java.util.Iterator it = tmp.iterator();
    while (it.hasNext()) {
      System.out.println(it.next());
    }
 */   
    return (javax.mail.Folder[])tmp.toArray(new javax.mail.Folder[tmp.size()]);
  }
  
/** Open this LocalFolder. This method is only valid on Folders that can
 * contain Messages and that are closed. <P>
 *
 * If this LocalFolder is opened successfully, an OPENED ConnectionEvent is
 * delivered to any ConnectionListener registered on this LocalFolder.
 * <P>
 *
 * @param mode open the LocalFolder READ_ONLY or READ_WRITE
 * @throws MessagingException -
 * @throws FolderNotFoundException if this Folder does not exist
 * @throws IllegalStateException if this LocalFolder is not in the closed state
 */  
  public void open(int mode) throws javax.mail.MessagingException,
  javax.mail.FolderNotFoundException, IllegalStateException {
    
    if (!exists()) throw new javax.mail.FolderNotFoundException();
    if (isOpen()) throw new IllegalStateException();
    
    isOpen = true;
    this.mode = mode;
    
    notifyConnectionListeners(javax.mail.event.ConnectionEvent.OPENED);
    
    // TODO: check for a lock for this folder and throw exception if it exists
    // TODO: read the index file and create the messages
  }
  
  public boolean renameTo(javax.mail.Folder folder) throws javax.mail.MessagingException {
    if (!exists()) throw new javax.mail.FolderNotFoundException();
    if (isOpen()) throw new IllegalStateException();
    
    // TODO: replace this simple renaming with renaming the folder on disk
    name.setName(folder.getName());
    return true;
  }
  
  public String toString() {
    return getFullName();
  }
}
